#!/usr/bin/perl
# Copyright (c) Sierra Wireless 2009
#
# Direct IP service removal perl script  1.0 Dec 22/2009
#
# 

use Foundation;
use POSIX;

$dip="<00a0d5ff ffaa>";   # ether address for direct IP 
$netInFile = "/Library/Preferences/SystemConfiguration/NetworkInterfaces.plist";
$prefFile = "/Library/Preferences/SystemConfiguration/preferences.plist";
$deleteKeys = 1;   # 1 to delete the direct IP keys, 0 otherwise


sub perlValue {
  my ( $object ) = @_;
  return $object->description()->UTF8String();
}

sub getPlistObject {
  my ( $object, @keysIndexes ) = ( @_ );

  if ( @keysIndexes ) {
    foreach my $keyIndex ( @keysIndexes ) {
      if ( $object and $$object ) {
       if ( $object->isKindOfClass_( NSArray->class ) ) 
        {
            $object = $object->objectAtIndex_( $keyIndex );
        } 
       elsif ( $object->isKindOfClass_( NSDictionary->class ) ) 
        {
            $object = $object->objectForKey_( $keyIndex );
        } 
        else 
        {
          print STDERR "Unknown type (not an array or a dictionary):\n";
          return;
        }
      } else {
        print STDERR "Got nil or other error for $keyIndex.\n";
        return;
      }
    }
  }
  return $object;
}


sub DeleteServiceInPreferences {
  my ( $reqDeviceName ) = @_;
  
  print "\tDeleteServiceInPreferences for DeviceName = $reqDeviceName\n";
  
  my $plist = NSDictionary->dictionaryWithContentsOfFile_( $prefFile );
  if ( $plist and $$plist) 
  {
    my $dico = getPlistObject( $plist, "NetworkServices" );
    if ( $dico and $$dico ) 
    {

    print "\t" . $prefFile . " NetworkServices dictionary count " . $dico->count. "\n";
    my $enumerator = $dico->keyEnumerator();

    while ($obj = $enumerator->nextObject() and $$obj) {
        my $key=perlValue($obj);
        #printf "######## %s\n", $key;
        my $deviceName=perlValue( getPlistObject( $plist, "NetworkServices", $key,"Interface" , "DeviceName" ));
      
        if ( $deviceName eq $reqDeviceName)
        {
            print "\t => key "  . $key . " found for "  . $reqDeviceName . " \n"; 
            if ( $deleteKeys eq 1 )
            {
            $result = `$PRIV "$PlistBuddy" -c "Delete :NetworkServices::$key dict" "$prefFile"`;
            printf "\tPlistBuddy delete NetworkServices result %d\n",$result;
            }
        }
   }
  }
  else
  {
    die "Error can not access NetworkServices from  n'$prefFile'\n";
    return;
  }
 }
 else
 {
    die  "Error loading file ($!): '$prefFile'\n";
    return;
  }
  return 0
}

################################ main ############################### 
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
printf "%4d-%02d-%02d %02d:%02d:%02d %s : to remove direct IP entries from Network preferences files\n", 
    $year+1900,$mon+1,$mday,$hour,$min,$sec, $0;

if ( ($#ARGV + 1) > 0 ) { $deleteKeys = 0; }

if (! -x ($PlistBuddy="/usr/libexec/PlistBuddy")) {
  if (! -x ($PlistBuddy="./PlistBuddy")) {
     die  "$0 Error: PlistBuddy $!\n";
   }
  }
 
 $PRIV="";
 if ( geteuid() != 0 ) {
	$PRIV="sudo";
	}

print "PlistBuddy '$PlistBuddy'  deleteKeys $deleteKeys PRIV '$PRIV'\n";
        
my $plist = NSDictionary->dictionaryWithContentsOfFile_( $netInFile );
if ( $plist and $$plist) 
{
  my $dico = getPlistObject( $plist, "Interfaces" );
  if ( $dico and $$dico ) 
  {
  print $netInFile . " Interfaces count " . $dico->count. "\n";

  for($i = 0; $i < $dico->count; $i++) 
  {
   my $object = $dico->objectAtIndex_( $i );
   if ( $object and $$object)		
     {
      $addr = getPlistObject( $plist, "Interfaces" ,$i, "IOMACAddress" );
      $macAddr=perlValue( $addr );
      $addr = getPlistObject( $plist, "Interfaces" ,$i, "BSD Name" );
      $bsdName=perlValue( $addr );
      print "Item ". ($i+1) . " IOMACAddress " .  $macAddr . "\t\tBSD name " . $bsdName;
      if ( $macAddr eq $dip)
      {
        print " 		=> Direct IP interface found\n"; 
        
        if ( $deleteKeys eq 1 )
         {
         my $result = `$PRIV "$PlistBuddy" -c "Delete :Interfaces::$i dict" "$netInFile"`;
         printf "PlistBuddy delete Interfaces result %d\n",$result;
         }

        DeleteServiceInPreferences($bsdName);
      }
      else {
        print "\n";
        }
     }
     else 
     {
        die "Could not find Index.\n";
     }
    }
   } 
   else 
   {
    die "Could not find Interfaces.\n";
    }
} 
else 
{
  die "Error loading file ($!): '$netInFile'\n";
}